<template>
  <gree-popup class="timer-picker-popup" v-model="isShow" position="bottom" get-container="body" @hide="hidePopup" safe-area-inset-bottom>
    <div class="timer-picker-popup">
      <!-- 标题 -->
      <div class="timer-picker-popup-title" v-text="$language('timer.title')" />
      <!-- 选择器 -->
      <gree-picker ref="picker" :columns="columns" :show-toolbar="false" v-if="isShow" />
      <!-- 底部 -->
      <div class="timer-picker-popup-footer">
        <!-- 小标题 -->
        <span class="timer-picker-popup-footer-title" v-html="`${$language('timer.repeat')}&nbsp&nbsp&nbsp|&nbsp;&nbsp; ${weekLoopText}`" />
        <!-- 日期按钮组 -->
        <div class="timer-picker-popup-footer-week">
          <!-- 按钮 -->
          <div
            class="timer-picker-popup-footer-week-btn"
            :class="{ select: week.select }"
            v-for="(week, weekIndex) in weekList"
            :key="weekIndex"
            v-text="week.text"
            @click="clickWeekBtn(weekIndex)"
          />
        </div>
        <!-- 确认框 -->
        <div class="timer-picker-popup-footer-box">
          <!-- 取消 -->
          <div class="timer-picker-popup-footer-box-cancel" v-text="$language('timer.Dialog_cancelText')" @click="hidePopup" />
          <!-- 确定 -->
          <div class="timer-picker-popup-footer-box-confirm" v-text="$language('timer.Popup_confirm')" @click="confirmSet" />
        </div>
      </div>
    </div>
  </gree-popup>
</template>

<script>
import { Popup, Picker } from 'gree-ui';
import { mapState, mapMutations } from 'vuex';

export default {
  name: 'TimerPicker',
  components: {
    [Popup.name]: Popup,
    [Picker.name]: Picker
  },
  props: {
    timerTaskTime: {
      type: Object,
      default: () => {
        return {};
      }
    },
    timerTaskWeeks: {
      type: Array,
      default: () => {
        return [];
      }
    },
    timerID: {
      type: Number,
      default: 0
    }
  },
  data() {
    return {
      isShow: false,
      weekList: [],
      columns: [
        {
          values: [],
          defaultIndex: 0
        },
        {
          values: [],
          defaultIndex: 0
        },
        {
          values: [],
          defaultIndex: 0
        }
      ],
      // pickerDefaultIndex: [2, 2, 0],
      selectData: {
        hour: 0,
        minus: 0,
        type: 1
      }
    };
  },
  computed: {
    ...mapState('control', {
      timerPicker: state => state.popup.timerPicker
    }),
    weekLoopText() {
      let result = '';
      let loop = false;
      let allSelect = true;
      this.weekList.forEach(week => {
        allSelect = allSelect && week.select;
        loop = loop || week.select;
        week.select && (result += `${week.text}&nbsp;&nbsp;`);
      });
      allSelect && (result = window.myvm.$i18n.t('timer.every'));
      loop || (result = window.myvm.$i18n.t('timer.once'));
      return result;
    }
  },
  watch: {
    timerPicker: {
      handler(newVal) {
        this.isShow = newVal;
      },
      immediate: true
    },
    isShow(val) {
      if (val) {
        this.getSelectData();
      } else {
        this.hidePopup();
      }
    }
  },
  created() {
    // 生成Picker数据
    const colsHour = this.genTimes(24, index => {
      const value = this.padZero(0 + index);
      return `${value}`;
    });
    const colsMinute = this.genTimes(60, index => {
      const value = this.padZero(0 + index);
      return `${value}`;
    });
    const colsOnOff = [this.$t('timer.turn_on'), this.$t('timer.turn_off')];
    this.columns[0].values = colsHour;
    this.columns[1].values = colsMinute;
    this.columns[2].values = colsOnOff;
    // 生成星期数据
    this.weekList = ['Mon', 'Tue', 'Wed', 'Thu', 'Fir', 'Sat', 'Sun'].map((value, index) => {
      return { text: this.$t(`timer.${value}`), index: String(index + 1), select: false };
    });
  },
  methods: {
    ...mapMutations('control', {
      SetPopup: 'SET_POPUP'
    }),
    getSelectData() {
      this.selectData = { ...this.timerTaskTime };
      this.weekList.forEach(week => {
        week.select = this.timerTaskWeeks.includes(week.index);
      });
      this.columns[0].defaultIndex = this.selectData.hour;
      this.columns[1].defaultIndex = this.selectData.minus;
      this.columns[2].defaultIndex = this.selectData.type;
    },
    // 获取定时执行时间
    getTargetTime() {
      const weeks = this.weekList.filter(week => week.select).map(week => Number(week.index));
      const [hour, minus, type] = this.pickerDefaultIndex;
      let diffTime;
      const date = new Date();
      if (weeks.length) {
        // 轮询选择的日期，返回最近的结果
        diffTime = weeks
          .map(day => {
            return date.dateDiff({
              hour,
              minus,
              day: day % 7
            });
          })
          .sort((a, b) => a.time - b.time)[0];
      } else {
        // 返回当前时间与选择时间的差值
        diffTime = date.dateDiff({
          hour,
          minus
        });
      }
      const timeString = `${diffTime.day ? `${diffTime.day}天` : ''}${diffTime.hour ? `${diffTime.hour}小时` : ''}${diffTime.minus}分钟`;
      return `${timeString}后${type ? '关机' : '开机'}`;
    },
    clickWeekBtn(weekIndex) {
      this.weekList[weekIndex].select = !this.weekList[weekIndex].select;
    },
    hidePopup() {
      this.SetPopup({ timerPicker: false });
    },
    confirmSet() {
      this.$emit('confirm', [this.$refs.picker.getValues(), this.weekList, this.timerID]);
      this.hidePopup();
    },
    getPickerData(columnIndex, itemIndex) {
      this.$set(this.pickerDefaultIndex, columnIndex, itemIndex);
    },
    //数字补零
    padZero(num, targetLength = 2) {
      let str = num + '';
      while (str.length < targetLength) {
        str = '0' + str;
      }
      return str;
    },
    //生成Picker数据
    genTimes(n, iteratee) {
      let index = -1;
      const result = Array(n);

      while (++index < n) {
        result[index] = iteratee(index);
      }

      return result;
    }
  }
};
</script>

<style lang="scss">
.timer-picker-popup {
  width: 100%;
  background-color: #fff;
  &-title {
    height: 156px;
    line-height: 156px;
    width: 100%;
    text-align: center;
    font-size: 52px;
  }
  .gree-picker__columns {
    .gree-picker__frame {
      &::before {
        content: ':';
        position: absolute;
        top: 50%;
        left: 33.333333333333336%;
        transform: translateY(-50%);
        color: #77a7e0;
        font-size: 56px;
      }
    }
  }
  &-footer {
    height: auto;
    display: flex;
    flex-wrap: wrap;
    align-items: flex-end;
    border-top: 1px solid #f0f0f0;
    margin-top: 120px;
    &-title {
      font-size: 36px;
      padding: 48px 0 0 64px;
      color: #404657;
      &.select {
        background-color: #71acf1;
        color: #71acf1;
      }
    }
    &-week {
      $roundWidth: 120px;
      $padding: 52px;
      width: 100%;
      height: auto;
      margin-top: 48px;
      padding: 0 $padding;
      display: flex;
      justify-content: space-between;
      flex-wrap: nowrap;
      &-btn {
        transform: scale(1);
        height: $roundWidth;
        width: $roundWidth;
        line-height: $roundWidth;
        border-radius: 50%;
        font-size: 35px;
        text-align: center;
        background-color: rgba(155, 155, 155, 0.15);
        // border: 1px solid rgba(140, 144, 154, 0.15);
        &.select {
          background-color: #71acf1;
          color: #fff;
        }
      }
    }
    &-box {
      $confirmHeight: 180px;
      height: $confirmHeight;
      width: 100%;
      margin-top: 48px;
      border-top: 1px solid #f0f0f0;
      color: #71acf1;
      font-size: 44px;
      display: flex;
      justify-content: space-between;
      &-cancel {
        color: #404657;
        border-right: 1px solid #f0f0f0;
      }
      &-confirm {
        color: #71acf1;
      }
      div {
        width: 50%;
        height: 100%;
        text-align: center;
        line-height: $confirmHeight;
      }
    }
  }
}
</style>
